@@ -46,6 +46,7 @@
 #include "empathy-ui-utils.h"
 #include "empathy-accounts-dialog.h"
 #include "empathy-images.h"
+#include "empathy-new-message-dialog.h"
 
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
@@ -138,6 +139,19 @@ static void       status_icon_event_free          (StatusIconEvent        *event
 
 G_DEFINE_TYPE (EmpathyStatusIcon, empathy_status_icon, G_TYPE_OBJECT);
 
+static void
+status_icon_notify_use_nm_cb (EmpathyConf  *conf,
+			      const gchar *key,
+			      gpointer     user_data)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (user_data);
+	gboolean               use_nm;
+
+	if (empathy_conf_get_bool (conf, key, &use_nm)) {
+		empathy_idle_set_use_nm (priv->idle, use_nm);
+	}
+}
+
 static void
 empathy_status_icon_class_init (EmpathyStatusIconClass *klass)
 {
@@ -153,13 +167,11 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv;
 	GList                 *pendings, *l;
+	gboolean               use_nm;
 
 	priv = GET_PRIV (icon);
 
 	priv->icon = gtk_status_icon_new ();
-	priv->idle = empathy_idle_new ();
-	empathy_idle_set_auto_away (priv->idle, TRUE);
-	empathy_idle_set_auto_disconnect (priv->idle, TRUE);
 	priv->manager = empathy_contact_manager_new ();
 	priv->mc = empathy_mission_control_new ();
 	priv->text_filter = empathy_filter_new ("org.gnome.Empathy.ChatFilter",
@@ -168,6 +180,19 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 						MC_FILTER_PRIORITY_DIALOG,
 						MC_FILTER_FLAG_INCOMING);
 
+	/* Setup EmpathyIdle */
+	priv->idle = empathy_idle_new ();
+	empathy_conf_get_bool (empathy_conf_get (),
+			       EMPATHY_PREFS_USE_NM,
+			       &use_nm);
+	empathy_conf_notify_add (empathy_conf_get (),
+				 EMPATHY_PREFS_USE_NM,
+				 status_icon_notify_use_nm_cb,
+				 icon);
+	empathy_idle_set_auto_away (priv->idle, TRUE);
+	empathy_idle_set_use_nm (priv->idle, use_nm);
+
+
 	status_icon_create_menu (icon);
 	status_icon_idle_notify_cb (icon);
 
@@ -397,16 +422,15 @@ status_icon_set_visibility (EmpathyStatusIcon *icon,
 
 	priv = GET_PRIV (icon);
 
+	empathy_conf_set_bool (empathy_conf_get (),
+			      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, !visible);
+
 	if (!visible) {
 		empathy_window_iconify (priv->window, priv->icon);
-		empathy_conf_set_bool (empathy_conf_get (),
-				      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
 	} else {
 		GList *accounts;
 
 		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
-		empathy_conf_set_bool (empathy_conf_get (),
-				      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, FALSE);
 	
 		/* Show the accounts dialog if there is no enabled accounts */
 		accounts = mc_accounts_list_by_enabled (TRUE);
@@ -528,7 +552,7 @@ status_icon_new_message_cb (GtkWidget         *widget,
 
 	priv = GET_PRIV (icon);
 
-	//empathy_new_message_dialog_show (GTK_WINDOW (priv->window));
+	empathy_new_message_dialog_show (NULL);
 }
 
 static void
