@@ -38,6 +38,7 @@
 #include "empathy-debug.h"
 #include "empathy-utils.h"
 #include "empathy-contact-manager.h"
+#include "empathy-tp-group.h"
 
 #define DEBUG_DOMAIN "Utils"
 
@@ -198,82 +199,6 @@ empathy_strncasecmp (const gchar *s1,
 	return ret_val;
 }
 
-/* Stolen from telepathy-glib */
-gboolean
-empathy_strdiff (const gchar *left, const gchar *right)
-{
-  if ((NULL == left) != (NULL == right))
-    return TRUE;
-
-  else if (left == right)
-    return FALSE;
-
-  else
-    return (0 != strcmp (left, right));
-}
-
-/* Stolen from telepathy-glib */
-static inline gboolean
-_esc_ident_bad (gchar c, gboolean is_first)
-{
-  return ((c < 'a' || c > 'z') &&
-          (c < 'A' || c > 'Z') &&
-          (c < '0' || c > '9' || is_first));
-}
-
-/* Stolen from telepathy-glib */
-gchar *
-empathy_escape_as_identifier (const gchar *name)
-{
-  gboolean bad = FALSE;
-  size_t len = 0;
-  GString *op;
-  const gchar *ptr, *first_ok;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  for (ptr = name; *ptr; ptr++)
-    {
-      if (_esc_ident_bad (*ptr, ptr == name))
-        {
-          bad = TRUE;
-          len += 3;
-        }
-      else
-        len++;
-    }
-
-  /* fast path if it's clean */
-  if (!bad)
-    return g_strdup (name);
-
-  /* If strictly less than ptr, first_ok is the first uncopied safe character.
-   */
-  first_ok = name;
-  op = g_string_sized_new (len);
-  for (ptr = name; *ptr; ptr++)
-    {
-      if (_esc_ident_bad (*ptr, ptr == name))
-        {
-          /* copy preceding safe characters if any */
-          if (first_ok < ptr)
-            {
-              g_string_append_len (op, first_ok, ptr - first_ok);
-            }
-          /* escape the unsafe character */
-          g_string_append_printf (op, "_%02x", (unsigned char)(*ptr));
-          /* restart after it */
-          first_ok = ptr + 1;
-        }
-    }
-  /* copy trailing safe characters if any */
-  if (first_ok < ptr)
-    {
-      g_string_append_len (op, first_ok, ptr - first_ok);
-    }
-  return g_string_free (op, FALSE);
-}
-
 gboolean
 empathy_xml_validate (xmlDoc      *doc,
 		     const gchar *dtd_filename)
@@ -368,16 +293,21 @@ empathy_xml_node_find_child_prop_value (xmlNodePtr   node,
 guint
 empathy_account_hash (gconstpointer key)
 {
+	g_return_val_if_fail (MC_IS_ACCOUNT (key), 0);
+
 	return g_str_hash (mc_account_get_unique_name (MC_ACCOUNT (key)));
 }
 
 gboolean
 empathy_account_equal (gconstpointer a,
-		      gconstpointer b)
+		       gconstpointer b)
 {
 	const gchar *name_a;
 	const gchar *name_b;
 
+	g_return_val_if_fail (MC_IS_ACCOUNT (a), FALSE);
+	g_return_val_if_fail (MC_IS_ACCOUNT (b), FALSE);
+
 	name_a = mc_account_get_unique_name (MC_ACCOUNT (a));
 	name_b = mc_account_get_unique_name (MC_ACCOUNT (b));
 
@@ -421,7 +351,7 @@ empathy_inspect_handle (McAccount *account,
 	GArray          *handles;
 	gchar          **names;
 	gchar           *name;
-	GError          *error;
+	GError          *error = NULL;
 
 	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
 	g_return_val_if_fail (handle != 0, NULL);
@@ -462,3 +392,92 @@ empathy_inspect_handle (McAccount *account,
 	return name;
 }
 
+void
+empathy_call_contact (EmpathyContact *contact)
+{
+#ifdef HAVE_VOIP
+	MissionControl *mc;
+	McAccount      *account;
+	TpConn         *tp_conn;
+	gchar          *object_path;
+	const gchar    *bus_name;
+	TpChan         *new_chan;
+	EmpathyTpGroup *group;
+	GError         *error = NULL;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+	/* StreamedMedia channels must have handle=0 and handle_type=none.
+	 * To call a contact we have to add him in the group interface of the
+	 * channel. MissionControl will detect the channel creation and 
+	 * dispatch it to the VoIP chandler automatically. */
+
+	mc = empathy_mission_control_new ();
+	account = empathy_contact_get_account (contact);
+	tp_conn = mission_control_get_connection (mc, account, NULL);
+	/* FIXME: Should be async */
+	if (!tp_conn_request_channel (DBUS_G_PROXY (tp_conn),
+				      TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+				      TP_HANDLE_TYPE_NONE,
+				      0,
+				      FALSE,
+				      &object_path,
+				      &error)) {
+		empathy_debug (DEBUG_DOMAIN, 
+			      "Couldn't request channel: %s",
+			      error ? error->message : "No error given");
+		g_clear_error (&error);
+		g_object_unref (mc);
+		g_object_unref (tp_conn);
+		return;
+	}
+
+	bus_name = dbus_g_proxy_get_bus_name (DBUS_G_PROXY (tp_conn));
+	new_chan = tp_chan_new (tp_get_bus (),
+				bus_name,
+				object_path,
+				TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+				TP_HANDLE_TYPE_NONE,
+				0);
+
+	group = empathy_tp_group_new (account, new_chan);
+	empathy_tp_group_add_member (group, contact, "");
+
+	g_object_unref (group);
+	g_object_unref (mc);
+	g_object_unref (tp_conn);
+	g_object_unref (new_chan);
+	g_free (object_path);
+#endif
+}
+
+void
+empathy_chat_with_contact (EmpathyContact  *contact)
+{
+	MissionControl *mc;
+
+	mc = empathy_mission_control_new ();
+	mission_control_request_channel (mc,
+					 empathy_contact_get_account (contact),
+					 TP_IFACE_CHANNEL_TYPE_TEXT,
+					 empathy_contact_get_handle (contact),
+					 TP_HANDLE_TYPE_CONTACT,
+					 NULL, NULL);
+	g_object_unref (mc);
+}
+
+void
+empathy_chat_with_contact_id (McAccount *account, const gchar *contact_id)
+{
+	MissionControl *mc;
+
+	mc = empathy_mission_control_new ();
+	mission_control_request_channel_with_string_handle (mc,
+							    account,
+							    TP_IFACE_CHANNEL_TYPE_TEXT,
+							    contact_id,
+							    TP_HANDLE_TYPE_CONTACT,
+							    NULL, NULL);
+	g_object_unref (mc);
+}
+
